unit RXMain;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  ExeImage, StdCtrls, Buttons, ExtCtrls, ComCtrls, Menus, ImgList;

type
  TMainForm = class(TForm)
    StatusBar: TStatusBar;
    TreeViewPanel: TPanel;
    Panel1: TPanel;
    TreeView: TTreeView;
    ListViewPanel: TPanel;
    ListViewCaption: TPanel;
    FileOpenDialog: TOpenDialog;
    FileSaveDialog: TSaveDialog;
    MainMenu: TMainMenu;
    miFile: TMenuItem;
    miFileExit: TMenuItem;
    miFileSave: TMenuItem;
    miFileOpen: TMenuItem;
    Small: TImageList;
    Large: TImageList;
    Splitter: TSplitter;
    Viewers: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    ImageViewer: TImage;
    StringViewer: TMemo;
    ListView: TListView;
    HexViewer: TMemo;
    procedure FileExit(Sender: TObject);
    procedure FileOpen(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ListViewEnter(Sender: TObject);
    procedure SaveResource(Sender: TObject);
    procedure TreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure ViewersChanging(Sender: TObject;
      var AllowChange: Boolean);
  private
    FExeFile: TExeImage;
    procedure LoadResources(ResList: TResourceList; Node: TTreeNode);
    procedure DisplayResources;
    procedure UpdateViewPanel;
    procedure UpdateListView(ResList: TResourceList);
    procedure UpdateStatusLine(ResItem: TResourceItem);
  end;

var
  MainForm: TMainForm;

implementation

uses RXTypes;

{$R *.dfm}
{$R RXIMAGES.RES}

const
  itBitmap: TResType = ImgList.rtBitmap; // Reference for duplicate identifier
  ImageMap: array[TResourceType] of Byte = (2,4,5,3,2,2,2,2,2,2,2,2,2,2,2,2,2);
  ResFiltMap: array[TResourceType] of Byte = (1,4,2,3,1,1,1,1,1,1,1,1,1,1,1,1,1);

  SNoResSelected = 'No resource selected';
  SSaveFilter    = 'Other Resource (*.*)|*.*|Bitmaps (*.BMP)|*.BMP|'+
                   'Icons (*.ICO)|*.ICO|Cursor (*.CUR)|*.CUR';
  SOpenFilter    = 'Executable File Images (*.EXE;*.DLL)|*.EXE;*.DLL|'+
                   'All Files (*.*)|*.*';

{ Utility Functions }

procedure Error(const ErrMsg: string);
begin
  raise Exception.Create(ErrMsg);
end;

{ Non Event Handlers }

procedure TMainForm.LoadResources(ResList: TResourceList; Node: TTreeNode);
var
    Item: TResourceItem;
    i: Integer;
    CNode: TTreeNode;
begin
    for i := 0 to ResList.Count - 1 do
    begin
        Item := ResList[I]; 
        CNode := TreeView.Items.AddChildObject(Node, Item.Name, Item);
        if Item.IsList then
        begin
            CNode.SelectedIndex := 1;
            LoadResources(Item.List, CNode);
        end else
        begin
            CNode.ImageIndex := ImageMap[Item.ResType];
            CNode.SelectedIndex := CNode.ImageIndex;
        end;
    end;
end;

procedure TMainForm.DisplayResources;
begin
    ListView.Items.Clear;
    TreeView.Selected := nil;
    TreeView.Items.Clear;
    LoadResources(FExeFile.Resources, nil);
    TreeView.SetFocus;
    TreeView.Selected := TreeView.Items[0];
end;

procedure TMainForm.UpdateViewPanel;
var
    R: TResourceItem;
begin
    if TreeView.Visible and Assigned(TreeView.Selected) then
    begin
        R := TResourceItem(TreeView.Selected.Data);
        if R.IsList then
        begin
            UpdateListView(R.List);
        end
        else
        begin
            case R.ResType of
            rtBitmap, rtIconEntry, rtCursorEntry:
                begin
                    ImageViewer.Picture.Assign(R);
                    Viewers.ActivePageIndex := 1;
                end;
            rtString, rtMenu:
                begin
                    StringViewer.Lines.Assign(R);
                    StringViewer.SelStart := 0;
                    Viewers.ActivePageIndex := 2;
                end
            else
                begin
                    HexViewer.Lines.Assign(R);
                    Viewers.ActivePageIndex := 3;
                end;
            end;
        end;
        UpdateStatusLine(R);
    end;
end;

procedure TMainForm.UpdateListView(ResList: TResourceList);
var
    I: Integer;
    Item: TListItem;
    R: TResourceItem;
begin
    ListView.Items.Clear;
    for I := 0 to ResList.Count-1 do
    begin
        R := ResList[I];
        Item := ListView.Items.Add;
        Item.Data := R;
        Item.Caption := R.Name;
        Item.SubItems.Add(Format('%.7x', [R.Offset]));
        Item.SubItems.Add(Format('%.5x', [R.Size]));
        Item.ImageIndex := ImageMap[R.ResType];
    end;
    Viewers.ActivePageIndex := 0;
end;

procedure TMainForm.UpdateStatusLine(ResItem: TResourceItem);
begin
    if ResItem.IsList then
    begin
        ListViewCaption.Caption := ' '+TreeView.Selected.Text;
        StatusBar.Panels[0].Text := Format(' %d object(s)', [ListView.Items.Count]);
        StatusBar.Panels[1].Text := Format(' Offset: %x', [ResItem.Offset]);
    end
    else
    begin
        ListViewCaption.Caption := Format(' %s: %s', [ResItem.ResTypeStr, ResItem.Name]);
        StatusBar.Panels[0].Text := '';
        StatusBar.Panels[1].Text := Format(' Offset: %x  Size: %x', [ResItem.Offset, ResItem.Size]);
    end;
end;

{ Form Initialization }

procedure TMainForm.FormCreate(Sender: TObject);
begin
    FileOpenDialog.Filter := SOpenFilter;
    FileSaveDialog.Filter := SSaveFilter;
    Small.ResourceLoad(itBitmap, 'SmallImages', clOlive);
    Large.ResourceLoad(itBitmap, 'LargeImages', clOlive);
    Viewers.ActivePageIndex := 0;
    if (ParamCount > 0) and FileExists(ParamStr(1)) then
    begin
        Show;
        FExeFile := TExeImage.CreateImage(Self, ParamStr(1));
        DisplayResources;
    end;
end;

{ Menu Event Handlers }

procedure TMainForm.FileOpen(Sender: TObject);
var
    TmpExeFile: TExeImage;
begin
    if not FileOpenDialog.Execute then Exit;
    TmpExeFile := TExeImage.CreateImage(Self, FileOpenDialog.FileName);
    if Assigned(FExeFile) then FExeFile.Destroy;
    FExeFile := TmpExeFile;
    DisplayResources;
end;

procedure TMainForm.FileExit(Sender: TObject);
begin
    Close;
end;

procedure TMainForm.ListViewEnter(Sender: TObject);
begin
    if (ListView.Items.Count > 1) and (ListView.Selected = nil) then
    begin
        ListView.Selected := ListView.Items[0];
        ListView.ItemFocused := ListView.Selected;
    end;
end;

procedure TMainForm.SaveResource(Sender: TObject);
var
  ResItem: TResourceitem;

  function TreeViewResourceSelected: Boolean;
  begin
    Result := Assigned(TreeView.Selected) and
              Assigned(TreeView.Selected.Data) and
              not TResourceItem(TreeView.Selected.Data).IsList;
    if Result then ResItem := TResourceItem(TreeView.Selected.Data);
  end;

  function ListViewResourceSelected: Boolean;
  begin
    Result := Assigned(ListView.Selected) and
              Assigned(ListView.Selected.Data) and
              not TResourceItem(ListView.Selected.Data).IsList;
    if Result then ResItem := TResourceItem(ListView.Selected.Data);
  end;

begin
  if TreeViewResourceSelected or ListViewResourceSelected then
  with FileSaveDialog do
  begin
    FilterIndex := ResFiltMap[ResItem.ResType];
    if Execute then
      ResItem.SaveToFile(FileName)
  end
  else
    Error(SNoResSelected);
end;

procedure TMainForm.TreeViewChange(Sender: TObject; Node: TTreeNode);
begin
  UpdateViewPanel;
end;

procedure TMainForm.ViewersChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
    AllowChange := false;
end;

end.
